@@ -20,7 +20,11 @@ from .thermodb import (
     build_components_thermodb,
     build_component_thermodb_from_reference,
     check_and_build_component_thermodb,
-    ComponentThermoDB
+    check_and_build_components_thermodb,
+    build_mixture_thermodb_from_reference,
+    check_and_build_mixture_thermodb,
+    ComponentThermoDB,
+    MixtureThermoDB
 )
 
 __all__ = [
@@ -43,5 +47,9 @@ __all__ = [
     'CustomRef',
     'build_component_thermodb_from_reference',
     'check_and_build_component_thermodb',
-    'ComponentThermoDB'
+    'check_and_build_components_thermodb',
+    'build_mixture_thermodb_from_reference',
+    'check_and_build_mixture_thermodb',
+    'ComponentThermoDB',
+    'MixtureThermoDB'
 ]
