@@ -369,6 +369,29 @@ class CompBuilder(CompExporter):
         except Exception as e:
             raise Exception('Checking properties failed!, ', e)
 
+    def is_property_available(
+        self,
+        thermo_name: str
+    ) -> bool:
+        '''
+        Check if a property is available in the thermodb
+
+        Parameters
+        ----------
+        thermo_name : str
+            name of the thermodynamic property
+
+        Returns
+        -------
+        res : bool
+            True if available
+        '''
+        try:
+            # check
+            return thermo_name in self.properties
+        except Exception as e:
+            raise Exception('Checking property availability failed!, ', e)
+
     def check_property(
         self,
         thermo_name: str
@@ -394,10 +417,10 @@ class CompBuilder(CompExporter):
 
     def select_property(
         self,
-            thermo_name: str
+        thermo_name: str
     ) -> TableData | TableMatrixData:
         '''
-        Select a thermodynamic property
+        Select a thermodynamic property (TableData or TableMatrixData) registered in the thermodb, case-sensitive.
 
         Parameters
         ----------
@@ -410,6 +433,23 @@ class CompBuilder(CompExporter):
             property registered in the thermodb
         '''
         try:
+            # NOTE: case-sensitive
+            # thermo_name normalize to lower case
+            thermo_name = thermo_name.strip().lower()
+
+            # SECTION: lookup
+            selected_property = None
+            for prop in self.properties.keys():
+                if prop.lower().strip() == thermo_name:
+                    selected_property = prop
+                    break
+
+            # NOTE: check
+            if selected_property is not None:
+                thermo_name = selected_property
+            else:
+                raise Exception('Property not found in the thermodb!')
+
             # check library
             return self.properties[thermo_name]
         except Exception as e:
@@ -430,6 +470,29 @@ class CompBuilder(CompExporter):
         except Exception as e:
             raise Exception('Checking functions failed!, ', e)
 
+    def is_function_available(
+        self,
+        function_name: str
+    ) -> bool:
+        '''
+        Check if a function is available in the thermodb
+
+        Parameters
+        ----------
+        function_name : str
+            name of the thermodynamic function
+
+        Returns
+        -------
+        res : bool
+            True if available
+        '''
+        try:
+            # check
+            return function_name in self.functions
+        except Exception as e:
+            raise Exception('Checking function availability failed!, ', e)
+
     def check_function(
         self,
         name: str
@@ -458,7 +521,7 @@ class CompBuilder(CompExporter):
         function_name: str
     ) -> TableEquation | TableMatrixEquation:
         '''
-        Select a function registered in the thermodb
+        Select a function registered in the thermodb (case-sensitive).
 
         Parameters
         ----------
@@ -469,8 +532,29 @@ class CompBuilder(CompExporter):
         -------
         TableMatrixEquation | TableEquation
             function registered in the thermodb
+
+        Notes
+        -----
+        - The search is case-sensitive.
         '''
         try:
+            # NOTE: case-sensitive
+            # functions normalize to lower case
+            function_name = function_name.strip().lower()
+
+            # SECTION: lookup
+            selected_function = None
+            for func in self.functions.keys():
+                if func.lower().strip() == function_name:
+                    selected_function = func
+                    break
+
+            # NOTE: check
+            if selected_function is not None:
+                function_name = selected_function
+            else:
+                raise Exception('Function not found in the thermodb!')
+
             # check library
             return self.functions[function_name]
         except Exception as e:
@@ -486,7 +570,7 @@ class CompBuilder(CompExporter):
         TableMatrixEquation
     ]:
         '''
-        Select a thermodynamic property or function registered in the thermodb
+        Select a thermodynamic property or function registered in the thermodb (case-sensitive).
 
         Parameters
         ----------
@@ -499,25 +583,49 @@ class CompBuilder(CompExporter):
             property defined in the thermodb
         '''
         try:
+            # SECTION: case-sensitive
+            # thermo_name normalize to lower case
+            thermo_name = thermo_name.strip()
+            # normalize to lower case
+            thermo_name_lower = thermo_name.lower()
+
             # SECTION 1: check if the property exists in both functions and properties
             check_list = list(self.functions.keys()) + \
                 list(self.properties.keys())
-            # check if the property exists in both functions and properties
-            if (thermo_name not in check_list):
+
+            # NOTE: normalize to lower case
+            check_list = [item.lower() for item in check_list]
+            # NOTE: check_list may contain duplicates
+            check_list = list(set(check_list))
+
+            # ! check if the property exists in both functions and properties
+            if (thermo_name_lower not in check_list):
                 # raise
                 raise Exception(
                     'Property exists in both functions and properties!')
 
-            # SECTION 2: check if the property is a function or a property
-            if thermo_name in self.functions:
-                # check if the property is a function
+            # SECTION: normalized names in functions and properties
+            # NOTE: functions
+            functions_keys_lower = [
+                k for k in self.functions.keys() if k.lower() == thermo_name_lower
+            ]
+            # check
+            if len(functions_keys_lower) == 1:
+                thermo_name = functions_keys_lower[0]
                 return self.functions[thermo_name]
-            elif thermo_name in self.properties:
-                # check if the property is a property
+
+            # NOTE: properties
+            properties_keys_lower = [
+                k for k in self.properties.keys() if k.lower() == thermo_name_lower
+            ]
+            # check
+            if len(properties_keys_lower) == 1:
+                thermo_name = properties_keys_lower[0]
                 return self.properties[thermo_name]
-            else:
-                # check if the property is a property in the thermodb
-                raise Exception('Property not found in the thermodb!')
+
+            # NOTE: not found
+            raise Exception('Property not found in the thermodb!')
+
         except Exception as e:
             raise Exception('Selecting a thermodynamic property failed!, ', e)
 
@@ -575,11 +683,13 @@ class CompBuilder(CompExporter):
                         f"Invalid source format! {property_source}")
                 # get property
                 prop = prop_src.get_property(
-                    source[1].strip(), message=message)
+                    source[1].strip(),
+                    message=message
+                )
                 # return
                 return prop
             elif isinstance(prop_src, TableMatrixData):
-                # NOTE: check strring format
+                # NOTE: check string format
                 if source_num == 2:
                     # property name full format
                     prop_name = source[1].strip()
@@ -593,23 +703,32 @@ class CompBuilder(CompExporter):
 
                     # NOTE: get property (ij method)
                     prop = prop_src.ij(
-                        prop_name, symbol_format=symbol_format, message=message)
+                        prop_name,
+                        symbol_format=symbol_format,
+                        message=message
+                    )
                     # return
                     return prop
                 elif source_num == 4:
                     # get components
                     component_names = source[2:]
                     # trim
-                    component_names = [name.strip()
-                                       for name in component_names]
+                    component_names = [
+                        name.strip() for name in component_names
+                    ]
                     # check length
                     if len(component_names) != 2:
                         raise ValueError(
-                            f"Invalid source format! {property_source}, components are required!")
+                            f"Invalid source format! {property_source}, components are required!"
+                        )
 
                     # NOTE: get property (get_matrix_property method)
-                    prop = prop_src.get_matrix_property(source[1].strip(), component_names=component_names,
-                                                        symbol_format=symbol_format, message=message)
+                    prop = prop_src.get_matrix_property(
+                        source[1].strip(),
+                        component_names=component_names,
+                        symbol_format=symbol_format,
+                        message=message
+                    )
                     # return
                     return prop
             else:
@@ -644,15 +763,18 @@ class CompBuilder(CompExporter):
             if filename is None:
                 raise Exception("Filename is required!")
 
-            # build
+            # NOTE: build
             self.build()
 
-            # file path setting
+            # NOTE: file path setting
             if file_path is None:
                 file_path = os.getcwd()
 
             # file full name
-            filename = os.path.join(file_path, filename)
+            filename = os.path.join(
+                file_path,
+                filename
+            )
 
             # file name path
             if not filename.endswith('.pkl'):
