@@ -1,5 +1,5 @@
 from .constants import __version__, __author__, __description__, __email__
-from .setting import API_URL, REFERENCE_CONFIG_KEYS
+from .setting import API_URL, REFERENCE_CONFIG_KEYS, DEFAULT_COMPONENT_STATES
 
 __all__ = [
     '__version__',
@@ -7,5 +7,6 @@ __all__ = [
     '__description__',
     '__email__',
     'API_URL',
-    'REFERENCE_CONFIG_KEYS'
+    'REFERENCE_CONFIG_KEYS',
+    'DEFAULT_COMPONENT_STATES'
 ]
