@@ -3,10 +3,12 @@ from .tabledata import TableData
 from .tableequation import TableEquation
 from .tablematrixdata import TableMatrixData
 from .tablematrixequation import TableMatrixEquation
+from .table_util import TableUtil
 
 __all__ = [
     'TableData',
     'TableEquation',
     'TableMatrixData',
-    'TableMatrixEquation'
+    'TableMatrixEquation',
+    'TableUtil',
 ]
