@@ -5,8 +5,9 @@ import math
 import json
 from typing import Literal, Optional, List, Dict, Any
 # local
-from ..models import EquationResult
+from ..models import EquationResult, PropertyMatch, EquationRangeResult
 from ..utils import format_eq_data
+from .table_util import TableUtil
 
 # NOTE: logger
 logger = logging.getLogger(__name__)
@@ -362,6 +363,23 @@ class TableEquation:
             logger.error(f'Loading error {e}!')
             return []
 
+    def get_arg_symbols(self):
+        '''Get argument symbols.'''
+        try:
+            # get arg symbols
+            _arg_symbols = self.arg_symbols
+
+            # extract symbols
+            symbols = []
+
+            # iterate through arg_symbols
+            for key, value in _arg_symbols.items():
+                symbols.append(value['symbol'])
+
+            return symbols
+        except Exception as e:
+            raise Exception(f'Loading error {e}!')
+
     def eq_info(self):
         '''Get equation information.'''
         try:
@@ -369,13 +387,123 @@ class TableEquation:
             _return = self.returns
 
             # check length
-            if len(_return) == 1:
+            # for dict
+            if isinstance(_return, dict):
                 return list(_return.values())[0]
+            elif isinstance(_return, list):
+                return _return[0]
+
+            # if len(_return) == 1:
+            #     return list(_return.values())[0]
             else:
                 raise Exception("Every equation has only one return")
         except Exception as e:
             raise Exception(f'Loading error {e}!')
 
+    def get_variable_range_values(
+            self,
+    ):
+        '''
+        Get variable range values for given variable names.
+
+        Parameters
+        ----------
+        None
+
+        Returns
+        -------
+        Dict[str, List[float]]
+            A dictionary with variable names as keys and their range values as lists.
+
+        Examples
+        --------
+        >>> res = get_variable_range_values(variable_names=['T', 'P'])
+        >>> print(res)
+
+        ```python
+            {
+            'T': [100.0, 500.0],
+            'P': [1.0, 10.0]
+            }
+        ```
+        '''
+        try:
+            def parse_min_max(var_name: str) -> dict:
+                if var_name.endswith("min"):
+                    return {"base": var_name[:-3], "type": "min"}
+                elif var_name.endswith("max"):
+                    return {"base": var_name[:-3], "type": "max"}
+                else:
+                    return {"base": var_name, "type": None}
+
+            # SECTION: table structure
+            table_structure = self.table_structure
+
+            # check table structure
+            if table_structure is None:
+                raise Exception('Table structure not defined!')
+
+            # SECTION: get range names in the table structure
+            # NOTE: symbol
+            symbols = table_structure.get('SYMBOL', [])
+            # >> check
+            if not symbols:
+                logger.error('No symbols found in table structure!')
+                return {}
+
+            # NOTE: return variable names
+            args_symbols = self.get_arg_symbols()
+
+            # >> check
+            if not args_symbols:
+                logger.error('No argument symbols found!')
+                return {}
+
+            # NOTE: get valid variable range names
+            valid_variables = TableUtil.get_variable_range(
+                variable_names=args_symbols,
+                symbol_names=symbols
+            )
+            # >> check
+            if not valid_variables:
+                logger.error('No valid variable range names found!')
+                return {}
+
+            # SECTION: extract variable range values
+            # NOTE: table data
+            # ! values for the component
+            data = self.trans_data
+
+            # res
+            variable_ranges = {}
+
+            # iterate through variable names
+            for var_name, var_range_names in valid_variables.items():
+                # create key
+                variable_ranges[var_name] = {}
+
+                # iterate through range names
+                for range_name in var_range_names:
+                    # extract range data
+                    dt_ = data.get(range_name, None)
+
+                    if dt_ is None:
+                        logger.warning(
+                            f'No data found for range name: {range_name}!'
+                        )
+                        continue
+
+                    # find min and max in range_name
+                    parse_key = parse_min_max(range_name)
+                    key_id = parse_key['type']
+
+                    variable_ranges[var_name][key_id] = dt_
+
+            return variable_ranges
+        except Exception as e:
+            logger.error(f'Loading error {e}!')
+            return {}
+
     def cal(
         self,
         message: str = '',
@@ -442,6 +570,116 @@ class TableEquation:
             logger.error(f'Calculation error {e}!')
             raise Exception(f'Calculation error {e}!')
 
+    def cal_range(
+        self,
+        variable_id: str,
+        variable_range_values: List[float],
+        message: str = '',
+        decimal_accuracy: int = 4,
+        **args,
+    ) -> EquationRangeResult:
+        """
+        Calculate over a specified range for a given variable.
+
+        Parameters
+        ----------
+        variable_id : str
+            The name or symbol of the variable to calculate over.
+        variable_range_values : List[float]
+            A list containing the range values for the variable.
+        message : str
+            A message to be printed.
+        decimal_accuracy : int
+            Decimal accuracy (default is 4).
+        args : dict
+            A dictionary containing other variable names and values.
+
+        Returns
+        -------
+        res: EquationRangeResult
+            Calculation results over the specified range.
+            - x: List of variable values.
+            - y: List of calculated results.
+            - name: Name of the calculated property.
+            - unit: Unit of the calculated property.
+            - symbol: Symbol of the calculated property.
+            - databook_name: Name of the databook.
+            - table_name: Name of the table.
+            - message: Message associated with the calculation.
+        """
+        try:
+            # SECTION: validate inputs
+            if not variable_id:
+                raise Exception('Variable name not defined!')
+
+            if not variable_range_values or len(variable_range_values) < 2:
+                raise Exception('Variable range values not properly defined!')
+
+            # NOTE: check variable availability
+            # variable symbol
+            variable_symbol = None
+
+            # iterate over args_symbols
+            for arg_k, arg_v in self.arg_symbols.items():
+                # check both name and symbol
+                if variable_id == arg_v['symbol']:
+                    # found
+                    variable_symbol = arg_v['symbol']
+                    break
+                elif variable_id == arg_v['name']:
+                    # found
+                    variable_symbol = arg_v['symbol']
+                    break
+
+            # >> check
+            if variable_symbol is None:
+                raise Exception(f'Variable {variable_id} not found!')
+
+            # SECTION: calculation over range
+            # init res
+            res = {
+                'x': [],
+                'y': [],
+                'name': '',
+                'unit': '',
+                'symbol': '',
+                'databook_name': self.databook_name,
+                'table_name': self.table_name,
+                'message': message or 'No message'
+            }
+
+            # NOTE: loop over range values
+            for var_value in variable_range_values:
+                # build args
+                calc_args = args.copy()
+                calc_args[variable_symbol] = var_value
+
+                # perform calculation
+                res_ = self.cal(
+                    message=message,
+                    decimal_accuracy=decimal_accuracy,
+                    **calc_args
+                )
+
+                # store result
+                res['x'].append(var_value)
+                res['y'].append(res_['value'])
+
+            # set name, unit, symbol
+            res['unit'] = res_['unit']
+            res['symbol'] = res_['symbol']
+            res['name'] = res_.get('name', None) or res_.get(
+                'equation_name', None)
+
+            # NOTE: convert to EquationRangeResult
+            res = EquationRangeResult(**res)
+
+            # res
+            return res
+        except Exception as e:
+            logger.error(f'Calculation over range error {e}!')
+            raise Exception(f'Calculation over range error {e}!')
+
     def cal_integral(self, **args):
         '''
         Calculate integral
@@ -999,3 +1237,128 @@ class TableEquation:
             return {}
         except Exception as e:
             raise Exception("Making return symbols failed!, ", e)
+
+    def is_symbol_available(self, symbol: str):
+        '''
+        Check if a symbol is available in the table data. This method is case-insensitive.
+
+        Parameters
+        ----------
+        symbol : str
+            Symbol to check.
+
+        Returns
+        -------
+        bool
+            True if the symbol is available, False otherwise.
+        '''
+        try:
+            # NOTE: get symbols
+            symbols = self.table_symbols
+
+            # SECTION: check if symbol exists (case-sensitive)
+            return TableUtil.is_symbol_available(symbol, symbols)
+        except Exception as e:
+            logger.error(f"Error checking symbol availability: {e}")
+            return PropertyMatch(
+                prop_id=symbol,
+                availability=False,
+                search_mode='SYMBOL',
+            )
+
+    def is_column_name_available(self, column_name: str):
+        '''
+        Check if a column name is available in the table data.
+
+        Parameters
+        ----------
+        column_name : str
+            Column name to check.
+
+        Returns
+        -------
+        bool
+            True if the column name is available, False otherwise.
+        '''
+        try:
+            # NOTE: get column names
+            column_names = self.table_columns
+
+            # SECTION: check if column exists (case-sensitive)
+            return TableUtil.is_column_name_available(column_name, column_names)
+        except Exception as e:
+            logger.error(f"Error checking column name availability: {e}")
+            return PropertyMatch(
+                prop_id=column_name,
+                availability=False,
+                search_mode='COLUMN',
+            )
+
+    def is_property_available(
+            self,
+            prop_id: str,
+            search_mode: Literal['SYMBOL', 'COLUMN', 'BOTH'] = 'BOTH'
+    ) -> PropertyMatch:
+        '''
+        Check if a property is available in the table data.
+
+        Parameters
+        ----------
+        prop_id : str
+            Property ID to check.
+        search_mode : Literal['SYMBOL', 'COLUMN', 'BOTH'], optional
+            Search mode (default: 'BOTH'). Can be 'SYMBOL', 'COLUMN', or 'BOTH'.
+
+        Returns
+        -------
+        bool
+            True if the property is available, False otherwise.
+        '''
+        try:
+            # NOTE: check inputs
+            if not isinstance(prop_id, str):
+                logger.error(
+                    "Invalid property ID input! Property ID must be a string.")
+                return False
+
+            # SECTION: get property names
+            if search_mode == 'SYMBOL':
+                # ! symbol only
+                return self.is_symbol_available(prop_id)
+            elif search_mode == 'COLUMN':
+                # ! column name only
+                return self.is_column_name_available(prop_id)
+            elif search_mode != 'BOTH':
+                logger.error(
+                    "Invalid search mode! Must be 'SYMBOL', 'COLUMN', or 'BOTH'."
+                )
+                return False
+
+            # SECTION: check both symbol and column name
+            # NOTE: check symbols (true/false)
+            check_symbol_res = self.is_symbol_available(prop_id)
+
+            # NOTE: check column names (true/false)
+            check_column_res = self.is_column_name_available(prop_id)
+
+            # check
+            if check_symbol_res.availability or check_column_res.availability:
+                return PropertyMatch(
+                    prop_id=prop_id,
+                    availability=True,
+                    search_mode='BOTH'
+                )
+            else:
+                return PropertyMatch(
+                    prop_id=prop_id,
+                    availability=False,
+                    search_mode='BOTH'
+                )
+
+        except Exception as e:
+            logger.error(f"Error checking property availability: {e}")
+            return PropertyMatch(
+                prop_id=prop_id,
+                availability=False,
+                search_mode=search_mode,
+            )
