@@ -3,7 +3,13 @@ from .content import ReferenceContent, load_custom_reference
 from .databook import ThermoDatabook
 from .reference import ThermoReference
 from .checker import ReferenceChecker
-from .main import check_custom_reference, load_reference_from_str
+from .main import (
+    check_custom_reference,
+    load_reference_from_str,
+    extract_reference_from_str,
+    load_default_symbols
+)
+from .reference_mapper import component_reference_mapper, mixture_reference_mapper
 
 __all__ = [
     "ReferenceConfig",
@@ -13,5 +19,9 @@ __all__ = [
     "ThermoReference",
     "ReferenceChecker",
     "check_custom_reference",
-    "load_reference_from_str"
+    "load_reference_from_str",
+    "component_reference_mapper",
+    "extract_reference_from_str",
+    "load_default_symbols",
+    "mixture_reference_mapper"
 ]
